@@ -1,19 +1,19 @@
 /* zxidmeta.c  -  Handwritten functions for metadata parsing and generation as well as CoT handling
- * Copyright (c) 2006-2008 Symlabs (symlabs@symlabs.com), All Rights Reserved.
+ * Copyright (c) 2006-2009 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * Author: Sampo Kellomaki (sampo@iki.fi)
  * This is confidential unpublished proprietary source code of the author.
  * NO WARRANTY, not even implied warranties. Contains trade secrets.
  * Distribution prohibited unless authorized in writing.
  * Licensed under Apache License 2.0, see file COPYING.
- * $Id: zxidmeta.c,v 1.45 2008-05-30 17:39:11 lbernardo Exp $
+ * $Id: zxidmeta.c,v 1.57 2009-09-07 16:13:02 sampo Exp $
  *
  * 12.8.2006,  created --Sampo
  * 12.10.2007, mild refactoring to process keys for xenc as well. --Sampo
  * 13.12.2007, fixed missing KeyDescriptor/@use as seen in CA IdP metadata --Sampo
  * 14.4.2008,  added SimpleSign --Sampo
+ * 7.10.2008,  added documentation --Sampo
  */
 
-#include <signal.h>
 #include <fcntl.h>
 #include <string.h>
 #include <stdio.h>
@@ -39,16 +39,9 @@
 #include "c/zx-ns.h"
 #include "c/zx-data.h"
 
-#define PEM_CERT_START  "-----BEGIN CERTIFICATE-----"
-#define PEM_CERT_END    "-----END CERTIFICATE-----"
-#define PEM_RSA_PRIV_KEY_START  "-----BEGIN RSA PRIVATE KEY-----"
-#define PEM_RSA_PRIV_KEY_END    "-----END RSA PRIVATE KEY-----"
-#define PEM_DSA_PRIV_KEY_START  "-----BEGIN DSA PRIVATE KEY-----"
-#define PEM_DSA_PRIV_KEY_END    "-----END DSA PRIVATE KEY-----"
-
 /* ============== CoT and Metadata of Others ============== */
 
-/* Process certificates (public keys) from a metadata for entity.
+/*() Process certificates (public keys) from a metadata for entity.
  * Since one entity can be both IdP and SP, this function may
  * be called twice per entity, with different kd argument. */
 
@@ -93,6 +86,17 @@ static void zxid_process_keys(struct zxid_conf* cf, struct zxid_entity* ent,
   }
 }
 
+/*() Parse Metadata, see [SAML2meta]. This function is quite low level
+ * and assumes it is processing a buffer (which may contain multiple
+ * instances of various metadata).
+ *
+ * cf:: ZXID configuration object, used here mainly for memory allocation
+ * md:: Value-result parameter. Pointer to char pointer pointing to the
+ *     beginning of the metadata. As metadata is scanned and parsed, this
+ *     pointer will be advanced
+ * lim:: End of the metadata buffer
+ * return:: Entity data structure composed from the metadata. */
+
 /* Called by:  zxid_get_ent_from_file, zxid_get_meta */
 struct zxid_entity* zxid_parse_meta(struct zxid_conf* cf, char** md, char* lim)
 {
@@ -141,7 +145,10 @@ struct zxid_entity* zxid_parse_meta(struct zxid_conf* cf, char** md, char* lim)
   return 0;
 }
 
-/* Called by:  opt, zxid_get_ent_ss */
+/*() Write metadata of an entity to the Circle of Trust (CoT) cache of
+ * the entity identified by cf. Mainly used by Auto-CoT. */
+
+/* Called by:  opt x3, zxid_get_ent_ss */
 int zxid_write_ent_to_cache(struct zxid_conf* cf, struct zxid_entity* ent)
 {
   struct zx_str* ss;
@@ -161,7 +168,17 @@ int zxid_write_ent_to_cache(struct zxid_conf* cf, struct zxid_entity* ent)
   return 1;
 }
 
-/* Called by:  main, zxid_get_ent_by_sha1_name, zxid_get_ent_from_cache */
+/*() Read metadata from a file.
+ *
+ * Usually the file will be named according to "sha1 name", which
+ * is safe base64 encoded SHA1 digest hash over the EntityID. This
+ * is used to ensure unique file name for each entity. However,
+ * this function will in fact read from any file name supplied.
+ *
+ * See also zxid_get_ent_from_cache() which will compute the sha1_name
+ * and then read the metadata. */
+
+/* Called by:  main x3, test_ibm_cert_problem_enc_dec, zxid_get_ent_by_sha1_name, zxid_get_ent_from_cache */
 struct zxid_entity* zxid_get_ent_from_file(struct zxid_conf* cf, char* sha1_name)
 {
   int n,got;
@@ -170,7 +187,7 @@ struct zxid_entity* zxid_get_ent_from_file(struct zxid_conf* cf, char* sha1_name
   char* p;
   struct zxid_entity* ent;
   
-  D("sha1_name(%s)", sha1_name);
+  DD("sha1_name(%s)", sha1_name);
   fd = open_fd_from_path(O_RDONLY, 0, "get_ent_from_file", "%s" ZXID_COT_DIR "%s", cf->path, sha1_name);
   if (fd == BADFD) {
     perror("open metadata to read");
@@ -180,7 +197,7 @@ struct zxid_entity* zxid_get_ent_from_file(struct zxid_conf* cf, char* sha1_name
   
   md_buf = ZX_ALLOC(cf->ctx, ZXID_MAX_MD+1);
   n = read_all_fd(fd, md_buf, ZXID_MAX_MD, &got);
-  D("==========sha1_name(%s)", sha1_name);
+  DD("==========sha1_name(%s)", sha1_name);
   if (n == -1) {
     perror("read metadata");
     D("Failed to read metadata for sha1_name(%s)", sha1_name);
@@ -191,7 +208,7 @@ struct zxid_entity* zxid_get_ent_from_file(struct zxid_conf* cf, char* sha1_name
   
   p = md_buf;
   ent = zxid_parse_meta(cf, &p, md_buf+got);
-  D("++++++++++++sha1_name(%s)", sha1_name);
+  DD("++++++++++++sha1_name(%s)", sha1_name);
   if (!ent) {
     ZX_FREE(cf->ctx, md_buf);
     ERR("***** Parsing metadata failed for sha1_name(%s)", sha1_name);
@@ -203,7 +220,10 @@ struct zxid_entity* zxid_get_ent_from_file(struct zxid_conf* cf, char* sha1_name
   return ent;
 }
 
-/* Called by:  zxid_get_ent_ss x2 */
+/*() Compute sha1_name for an entity and then read the metadata from
+* the CoT metadata cache directory, e.g. /var/zxit/cot. */
+
+/* Called by:  main x5, zxid_get_ent_ss x2 */
 struct zxid_entity* zxid_get_ent_from_cache(struct zxid_conf* cf, struct zx_str* eid)
 {
   struct zxid_entity* ent;
@@ -218,12 +238,19 @@ struct zxid_entity* zxid_get_ent_from_cache(struct zxid_conf* cf, struct zx_str*
   return zxid_get_ent_from_file(cf, sha1_name);
 }
 
-/* Called by:  zxid_get_ent, zxid_idp_loc_raw, zxid_sp_sso_finalize */
+/*(i) Get metadata for entity, either from cache or network (using WKL), depending
+ * on configuration options.
+ *
+ * cf:: ZXID configuration object
+ * eid:: Entity ID whose metadata is desired
+ * return:: Entity data structure, including the metadata */
+
+/* Called by:  zxid_chk_sig, zxid_decode_redir_or_post, zxid_get_ent, zxid_get_ses_idp, zxid_idp_dispatch, zxid_idp_sso, zxid_slo_resp_redir, zxid_sp_dispatch, zxid_sp_sso_finalize */
 struct zxid_entity* zxid_get_ent_ss(struct zxid_conf* cf, struct zx_str* eid)
 {
   struct zxid_entity* ent;
   
-  D("eid(%.*s) path(%.*s) cf->magic=%x, md_cache_first(%d), cot(%x)", eid->len, eid->s, cf->path_len, cf->path, cf->magic, cf->md_cache_first, cf->cot);
+  D("eid(%.*s) path(%.*s) cf->magic=%x, md_cache_first(%d), cot(%p)", eid->len, eid->s, cf->path_len, cf->path, cf->magic, cf->md_cache_first, cf->cot);
   if (cf->md_cache_first) {
     ent = zxid_get_ent_from_cache(cf, eid);
     if (ent)
@@ -252,7 +279,9 @@ struct zxid_entity* zxid_get_ent_ss(struct zxid_conf* cf, struct zx_str* eid)
   return 0;
 }
 
-/* Called by:  zxid_cdc_check, zxid_start_sso_url */
+/*() Wrapper for zxid_get_ent_ss(), which see. */
+
+/* Called by:  zxid_cdc_check x2, zxid_start_sso_url */
 struct zxid_entity* zxid_get_ent(struct zxid_conf* cf, char* eid)
 {
   struct zx_str ss;
@@ -260,10 +289,12 @@ struct zxid_entity* zxid_get_ent(struct zxid_conf* cf, char* eid)
     return 0;
   ss.s = eid;
   ss.len = strlen(eid);
-  D("eid: (%s)\n", eid);
+  DD("eid: (%s)", eid);
   return zxid_get_ent_ss(cf, &ss);
 }
 
+/*() Given sha1_name, check in memory cache and if not, the disk cache. Do not try net (WKL). */
+
 /* Called by:  zxid_get_ent_by_succinct_id, zxid_load_cot_cache */
 struct zxid_entity* zxid_get_ent_by_sha1_name(struct zxid_conf* cf, char* sha1_name)
 {
@@ -277,6 +308,10 @@ struct zxid_entity* zxid_get_ent_by_sha1_name(struct zxid_conf* cf, char* sha1_n
   return ent;
 }
 
+/*() In artifact profile concept of "succinct id" appears. If you have one of those,
+ * you canuse this function to fetch the entity metadata. Only in-memory
+ * and disk caches will be tried. No network connection (WKL) will be initiated. */
+
 /* Called by:  zxid_sp_deref_art */
 struct zxid_entity* zxid_get_ent_by_succinct_id(struct zxid_conf* cf, char* raw_succinct_id)
 {
@@ -286,13 +321,16 @@ struct zxid_entity* zxid_get_ent_by_succinct_id(struct zxid_conf* cf, char* raw_
   return zxid_get_ent_by_sha1_name(cf, sha1_name);
 }
 
-/* Usually you will want to use the get_ent() methods if you need
- * specific entities. Loading the entire cache is expansive and
+/*() Usually you will want to use the get_ent() methods if you need
+ * only specific entities. Loading the entire cache is expensive and
  * only useful if you really need to enumerate through all
- * available entities, as may be the case  when rendering login
- * buttons for all IdPs in a user interface. */
+ * available entities. This may be the case when rendering login
+ * buttons for all IdPs in a user interface.
+ *
+ * cf:: ZXID configuration object
+ * return:: Linked list of Entity objects (metadata) for CoT partners */
 
-/* Called by:  main, zxid_idp_list_cf, zxid_mk_idp_list */
+/* Called by:  main x2, zxid_idp_list_cf_cgi, zxid_mk_idp_list */
 struct zxid_entity* zxid_load_cot_cache(struct zxid_conf* cf)
 {
   struct dirent* de;
@@ -317,7 +355,7 @@ struct zxid_entity* zxid_load_cot_cache(struct zxid_conf* cf)
     if (de->d_name[0] != '.' && de->d_name[strlen(de->d_name)-1] != '~')
       zxid_get_ent_by_sha1_name(cf, de->d_name);
   
-  D("HERE %p", cf);
+  DD("HERE %p", cf);
   closedir(dir);
   return cf->cot;
 }
@@ -325,8 +363,10 @@ struct zxid_entity* zxid_load_cot_cache(struct zxid_conf* cf)
 /* ============== Our Metadata ============== */
 
 #if 0
-/* In the metadata the PEM start/end cert markers are NOT used. */
-/* Called by:  zxid_sp_sso_desc x2 */
+/*() Constructor for key descriptor.
+ * In the metadata the PEM start/end cert markers are NOT used. */
+
+/* Called by:  zxid_idp_sso_desc x2, zxid_sp_sso_desc x2 */
 struct zx_md_KeyDescriptor_s* zxid_key_desc(struct zxid_conf* cf, char* use, X509* x)
 {
   int len;
@@ -368,6 +408,9 @@ struct zx_md_KeyDescriptor_s* zxid_key_desc(struct zxid_conf* cf, char* use, X50
 }
 #endif
 
+/*() Generate XML-DSIG key info given X509 certificate. */
+
+/* Called by:  zxenc_pubkey_enc, zxid_key_desc */
 struct zx_ds_KeyInfo_s* zxid_key_info(struct zxid_conf* cf, X509* x)
 {
   int len;
@@ -398,7 +441,9 @@ struct zx_ds_KeyInfo_s* zxid_key_info(struct zxid_conf* cf, X509* x)
   return ki;
 }
 
-/* Called by:  zxid_sp_sso_desc x2 */
+/*() Generate key descriptor metadata fragment given X509 certificate [SAML2meta]. */
+
+/* Called by:  zxid_idp_sso_desc x2, zxid_sp_sso_desc x2 */
 struct zx_md_KeyDescriptor_s* zxid_key_desc(struct zxid_conf* cf, char* use, X509* x)
 {
   struct zx_md_KeyDescriptor_s* kd = zx_NEW_md_KeyDescriptor(cf->ctx);
@@ -407,7 +452,37 @@ struct zx_md_KeyDescriptor_s* zxid_key_desc(struct zxid_conf* cf, char* use, X50
   return kd;
 }
 
-/* Called by:  zxid_sp_sso_desc x2 */
+/*() Generate Artifact Resolution (AR) Descriptor idp metadata fragment [SAML2meta]. */
+
+/* Called by:  zxid_idp_sso_desc */
+struct zx_md_ArtifactResolutionService_s* zxid_ar_desc(struct zxid_conf* cf,
+						       char* binding, char* loc, char* resp_loc)
+{
+  struct zx_md_ArtifactResolutionService_s* d = zx_NEW_md_ArtifactResolutionService(cf->ctx);
+  d->Binding = zx_ref_str(cf->ctx, binding);
+  d->Location = zx_strf(cf->ctx, "%s%s", cf->url, loc);
+  if (resp_loc)
+    d->ResponseLocation = zx_strf(cf->ctx, "%s%s", cf->url, resp_loc);
+  return d;
+}
+
+/*() Generate Single SignOn (SSO) Descriptor idp metadata fragment [SAML2meta]. */
+
+/* Called by:  zxid_idp_sso_desc */
+struct zx_md_SingleSignOnService_s* zxid_sso_desc(struct zxid_conf* cf,
+						  char* binding, char* loc, char* resp_loc)
+{
+  struct zx_md_SingleSignOnService_s* d = zx_NEW_md_SingleSignOnService(cf->ctx);
+  d->Binding = zx_ref_str(cf->ctx, binding);
+  d->Location = zx_strf(cf->ctx, "%s%s", cf->url, loc);
+  if (resp_loc)
+    d->ResponseLocation = zx_strf(cf->ctx, "%s%s", cf->url, resp_loc);
+  return d;
+}
+
+/*() Generate Single Logout (SLO) Descriptor metadata fragment [SAML2meta]. */
+
+/* Called by:  zxid_idp_sso_desc x2, zxid_sp_sso_desc x2 */
 struct zx_md_SingleLogoutService_s* zxid_slo_desc(struct zxid_conf* cf,
 						  char* binding, char* loc, char* resp_loc)
 {
@@ -419,7 +494,9 @@ struct zx_md_SingleLogoutService_s* zxid_slo_desc(struct zxid_conf* cf,
   return d;
 }
 
-/* Called by:  zxid_sp_sso_desc x2 */
+/*() Generate Manage Name Id (MNI) Descriptor metadata fragment [SAML2meta]. */
+
+/* Called by:  zxid_idp_sso_desc x2, zxid_sp_sso_desc x2 */
 struct zx_md_ManageNameIDService_s* zxid_mni_desc(struct zxid_conf* cf,
 						  char* binding, char* loc, char* resp_loc)
 {
@@ -431,7 +508,9 @@ struct zx_md_ManageNameIDService_s* zxid_mni_desc(struct zxid_conf* cf,
   return d;
 }
 
-/* Called by:  zxid_sp_sso_desc x4 */
+/*() Generate Assertion Consumer Service (SSO) Descriptor metadata fragment [SAML2meta]. */
+
+/* Called by:  zxid_sp_sso_desc x5 */
 struct zx_md_AssertionConsumerService_s* zxid_ac_desc(struct zxid_conf* cf,
 						      char* binding, char* loc, char* ix)
 {
@@ -442,6 +521,8 @@ struct zx_md_AssertionConsumerService_s* zxid_ac_desc(struct zxid_conf* cf,
   return d;
 }
 
+/*() Generate SP SSO Descriptor metadata fragment [SAML2meta]. */
+
 /* Called by:  zxid_sp_meta */
 struct zx_md_SPSSODescriptor_s* zxid_sp_sso_desc(struct zxid_conf* cf)
 {
@@ -524,7 +605,91 @@ struct zx_md_SPSSODescriptor_s* zxid_sp_sso_desc(struct zxid_conf* cf)
   return sp_ssod;
 }
 
-/* Called by:  main, zxid_idp_select, zxid_mk_ecp_Request_hdr, zxid_my_issuer, zxid_sp_meta, zxid_sp_sso_finalize */
+/*() Generate IdP SSO Descriptor metadata fragment [SAML2meta]. */
+
+/* Called by:  zxid_sp_meta */
+struct zx_md_IDPSSODescriptor_s* zxid_idp_sso_desc(struct zxid_conf* cf)
+{
+  struct zx_md_ArtifactResolutionService_s* z5;
+  struct zx_elem_s* ze;
+  struct zx_md_ManageNameIDService_s* z3;
+  struct zx_md_SingleLogoutService_s* z2;
+  struct zx_md_SingleSignOnService_s* z4;
+  struct zx_md_KeyDescriptor_s* zk;
+  struct zx_md_IDPSSODescriptor_s* idp_ssod = zx_NEW_md_IDPSSODescriptor(cf->ctx);
+  idp_ssod->WantAuthnRequestsSigned    = zx_ref_str(cf->ctx, cf->want_authn_req_signed?"1":"0");
+  idp_ssod->errorURL                   = zx_strf(cf->ctx, "%s?o=E", cf->url);
+  idp_ssod->protocolSupportEnumeration = zx_ref_str(cf->ctx, SAML2_PROTO);
+
+  if (!cf->enc_cert)
+    cf->enc_cert = zxid_read_cert(cf, "enc-nopw-cert.pem");
+
+  if (!cf->sign_cert)
+    cf->sign_cert = zxid_read_cert(cf, "sign-nopw-cert.pem");
+
+  if (!cf->enc_cert || !cf->sign_cert) {
+    ERR("Signing or encryption certificate not found (or both are corrupt). %p", cf->enc_cert);
+  } else {
+    zk = zxid_key_desc(cf, "encryption", cf->enc_cert);
+    zk->gg.g.n = &idp_ssod->KeyDescriptor->gg.g;
+    idp_ssod->KeyDescriptor = zk;
+    
+    zk = zxid_key_desc(cf, "signing", cf->sign_cert);
+    zk->gg.g.n = &idp_ssod->KeyDescriptor->gg.g;
+    idp_ssod->KeyDescriptor = zk;
+  }
+
+#if 0
+  /* *** NI */
+  z5 = zxid_ar_desc(cf, SAML2_SOAP, "?o=S", 0);
+  z5->gg.g.n = &idp_ssod->ArtifactResolutionService->gg.g;
+  idp_ssod->ArtifactResolutionService = z5;
+#endif
+
+  z2 = zxid_slo_desc(cf, SAML2_REDIR, "?o=Q", "?o=Q");
+  z2->gg.g.n = &idp_ssod->SingleLogoutService->gg.g;
+  idp_ssod->SingleLogoutService = z2;
+
+  z2 = zxid_slo_desc(cf, SAML2_SOAP, "?o=S", 0);
+  z2->gg.g.n = &idp_ssod->SingleLogoutService->gg.g;
+  idp_ssod->SingleLogoutService = z2;
+
+#if 0
+  /* *** NI */
+  z3 = zxid_mni_desc(cf, SAML2_REDIR, "?o=Q", "?o=Q");
+  z3->gg.g.n = &idp_ssod->ManageNameIDService->gg.g;
+  idp_ssod->ManageNameIDService = z3;
+
+  z3 = zxid_mni_desc(cf, SAML2_SOAP, "?o=S", 0);
+  z3->gg.g.n = &idp_ssod->ManageNameIDService->gg.g;
+  idp_ssod->ManageNameIDService = z3;
+#endif
+
+  ze = zx_ref_simple_elem(cf->ctx, SAML2_PERSISTENT_NID_FMT);
+  ze->g.n = &idp_ssod->NameIDFormat->g;
+  idp_ssod->NameIDFormat = ze;
+
+  ze = zx_ref_simple_elem(cf->ctx, SAML2_TRANSIENT_NID_FMT);
+  ze->g.n = &idp_ssod->NameIDFormat->g;
+  idp_ssod->NameIDFormat = ze;
+
+  z4 = zxid_sso_desc(cf, SAML2_REDIR, "?o=F", 0);
+  z4->gg.g.n = &idp_ssod->SingleSignOnService->gg.g;
+  idp_ssod->SingleSignOnService = z4;
+
+  return idp_ssod;
+}
+
+/*(i) Primary interface to our own Entity ID. While this would usually be
+ * automatically generated from URL configuration option so as to conform
+ * to the Well Known Location (WKL) metadata exchange convention [SAML2meta],
+ * on some sites the entity ID may be different and thus everybody who
+ * does not know better should use this interface to obtain it.
+ *
+ * cf:: ZXID configuration object, used to compute EntityID and also for memory allocation
+ * return:: Entity ID as zx_str */
+
+/* Called by:  main x2, zxid_an_page_cf, zxid_idp_select_zxstr_cf_cgi, zxid_mk_ecp_Request_hdr, zxid_mk_subj, zxid_my_issuer, zxid_sp_meta, zxid_sp_sso_finalize, zxid_validate_conditions, zxid_wsc_call */
 struct zx_str* zxid_my_entity_id(struct zxid_conf* cf)
 {
   if (cf->non_standard_entityid) {
@@ -536,12 +701,18 @@ struct zx_str* zxid_my_entity_id(struct zxid_conf* cf)
   }
 }
 
+/*() Dynamically determine our Common Domain Cookie (IdP discovery) URL. */
+
 /* Called by: */
 struct zx_str* zxid_my_cdc_url(struct zxid_conf* cf)
 {
   return zx_strf(cf->ctx, "%s?o=C", cf->cdc_url);
 }
 
+/*() Generate Issuer value. Issuer is often same as Entity ID, but sometimes
+ * it will be affiliation ID. This function is a low level interface. Usually
+ * you would want to use zxid_my_issuer(). */
+
 /* Called by:  zxid_my_issuer */
 struct zx_sa_Issuer_s* zxid_issuer(struct zxid_conf* cf, struct zx_str* nameid, char* affiliation)
 {
@@ -553,12 +724,17 @@ struct zx_sa_Issuer_s* zxid_issuer(struct zxid_conf* cf, struct zx_str* nameid,
   return is;
 }
 
-/* Called by:  zxid_mk_art_deref x2, zxid_mk_authn_req, zxid_mk_ecp_Request_hdr, zxid_mk_logout x2, zxid_mk_logout_resp x2, zxid_mk_mni, zxid_mk_mni_resp */
+/*() Generate Issuer value for our entity. Issuer is often same as Entity ID, but sometimes
+ * it will be affiliation ID. */
+
+/* Called by:  zxid_mk_a7n, zxid_mk_art_deref, zxid_mk_authn_req, zxid_mk_ecp_Request_hdr, zxid_mk_logout, zxid_mk_logout_resp, zxid_mk_mni, zxid_mk_mni_resp, zxid_mk_saml_resp */
 struct zx_sa_Issuer_s* zxid_my_issuer(struct zxid_conf* cf) {
   return zxid_issuer(cf, zxid_my_entity_id(cf), cf->affiliation);
 }
 
-/* Called by:  zxid_send_sp_meta, zxid_simple_cf */
+/*() Generate our SP metadata and return it as a string. */
+
+/* Called by:  zxid_send_sp_meta, zxid_simple_show_meta */
 struct zx_str* zxid_sp_meta(struct zxid_conf* cf, struct zxid_cgi* cgi)
 {
   struct zx_md_EntityDescriptor_s* ed;
@@ -566,22 +742,73 @@ struct zx_str* zxid_sp_meta(struct zxid_conf* cf, struct zxid_cgi* cgi)
   ed = zx_NEW_md_EntityDescriptor(cf->ctx);
   ed->entityID = zxid_my_entity_id(cf);
   ed->SPSSODescriptor = zxid_sp_sso_desc(cf);
+  if (cf->idp_ena)
+    ed->IDPSSODescriptor = zxid_idp_sso_desc(cf);
   
   if (cf->log_level>0)
     zxlog(cf, 0, 0, 0, 0, 0, 0, 0, "N", "W", "MYMD", 0, 0);
   return zx_EASY_ENC_SO_md_EntityDescriptor(cf->ctx, ed);
 }
 
-/* Called by:  main, opt */
+/*() Generate our SP metadata and send it to remote partner.
+ *
+ * Limitation:: This function only works with CGI as it will print the
+ *     serialized metadata straight to stdout. There are other
+ *     methods for getting metadat without this limitation, e.g. zxid_sp_meta() */
+
+/* Called by:  main x2, opt x2 */
 int zxid_send_sp_meta(struct zxid_conf* cf, struct zxid_cgi* cgi)
 {
   struct zx_str* ss = zxid_sp_meta(cf, cgi);
   if (!ss)
     return 0;
   //write_all_fd(1, ss->s, ss->len);
-  write_all_fd(stdout, ss->s, ss->len);
+  write_all_fd(fileno(stdout), ss->s, ss->len);
   zx_str_free(cf->ctx, ss);
   return 0;
 }
 
+/* ------- CARML ------- */
+
+/*() Generate our SP CARML and return it as a string. */
+
+struct zx_str* zxid_sp_carml(struct zxid_conf* cf)
+{
+  if (cf->log_level>0)
+    zxlog(cf, 0, 0, 0, 0, 0, 0, 0, "N", "W", "MYCARML", 0, 0);
+
+  /* *** Much work needed to study CARML spec and to convert need and want to comply */
+
+  return zx_strf(cf->ctx,
+"<carml:ClientAttrReq"
+" AppName=\"ZXID SP\""
+" Description=\"ZXID SP Attribute Needs and Wants\""
+" xmlns:carml=\"urn:igf:client:0.9:carml\""
+"<carml:DataDefs>"
+
+"  <carml:Attributes>"
+"  </carml:Attributes>"
+
+"  <carml:Predicates>"
+"  </carml:Predicates>"
+
+"  <carml:Roles>"
+"  </carml:Roles>"
+
+"  <carml:Policies>"
+"  </carml:Policies>"
+
+"</carml:DataDefs>"
+
+"<carml:ReadInteraction/>"
+"<carml:FindInteraction/>"
+"<carml:SearchInteraction/>"
+"<carml:CompareInteraction/>"
+"<carml:ModifyInteraction/>"
+"<carml:AddInteraction/>"
+
+"</carml:ClientAttrReq>"
+		 );
+}
+
 /* EOF  --  zxidmeta.c */
